from selenium import webdriver
from typing import List, Dict
from data_util import Data, DataUsage
import selenium_util
import logging


class CDHNavgatorScraper:
    def __init__(self, driver_path: str, timeout: int, logger: logging.Logger):
        self.driver = webdriver.Chrome(driver_path)
        self.driver.implicitly_wait(10)
        self.timeout = timeout
        self.logger = logger

    def login(self, url: str, username: str, password: str):
        self.driver.get(f"{url}/login.html")
        elems = selenium_util.find_elements_by_name(self.driver, "j_username", "j_password", "submit")
        elems["j_username"].send_keys(username)
        elems["j_password"].send_keys(password)
        elems["submit"].click()

    def get_search_result_count(self, url: str) -> int:
        self.driver.get(url)
        selector = "span.search-results-count-total-num"
        elem = selenium_util.find_element_waiting_explicitly(self.driver, self.timeout, selector)
        return int(elem.text.replace(",", ""))

    def scrape_search_results(self, url: str) -> List[str]:
        result = []
        self.driver.get(url)
        selector = ".search-result-item > div.search-result-item-prop-container > header.flex > a.entity"
        elems = selenium_util.find_elements_waiting_explicitly(self.driver, self.timeout, selector)
        for elem in elems:
            result.append(elem.get_attribute("href"))
        return result

    def scrape_operation_executions(self, urls: List[str]) -> Dict[str, int]:
        result = {}
        idx = 0
        count_selector = "div.details-page-columns div:nth-child(8) span.count span"
        selector = "a.details-page-entity-list-item-name"
        for url in urls:
            idx += 1
            self.logger.info(f"scraping operation execution... [{idx}/{len(urls)}]")
            self.driver.get(url)
            count = selenium_util.find_element_waiting_explicitly(self.driver, self.timeout, count_selector)
            if count.text != "0":
                elems = selenium_util.find_elements_waiting_explicitly(self.driver, self.timeout, selector)
                for elem in elems:
                    table_url = elem.get_attribute("href")
                    if table_url not in result:
                        result[table_url] = 0
                    result[table_url] = result[table_url] + 1
        return result

    def scrape_table_info(self, url_and_usage: Dict[str, int], data_usage: DataUsage, pattern: str) -> DataUsage:
        for i, table_url in enumerate(url_and_usage.keys()):
            self.logger.info(f"scraping table info [{i+1}/{len(url_and_usage)}]")
            self.driver.get(table_url)
            usage = url_and_usage[table_url]
            database_name = selenium_util.find_element_waiting_explicitly(
                self.driver, self.timeout, "value > a"
            ).text.strip()
            table_name = selenium_util.find_element_waiting_explicitly(
                self.driver, self.timeout, "h1.nav-page-header-title"
            ).text.strip()
            if database_name == "snapshot" or database_name == "compact":
                data = Data.from_pattern(pattern, f"{database_name}.{table_name}")
                data_usage.set_usage_with_data(usage, data)
                total = data_usage.get_usage(data.service, data.version, data.module, "total")
                data_usage.set_usage(total+usage, data.service, data.version, data.module, "total")
        return data_usage

    def close(self):
        self.driver.close()
