import re


class Data:

    def __init__(self, service: str, version: str, module: str, data_type: str):
        self.service = service
        self.version = version
        self.module = module
        self.data_type = data_type

    @staticmethod
    def from_pattern(pattern: str, string: str):
        return Data.from_dict(re.search(pattern, string).groupdict())

    @staticmethod
    def from_dict(data_dict):
        service = data_dict["service"] if "service" in data_dict else "no_service"
        version = data_dict["version"] if "version" in data_dict else "no_version"
        module = data_dict["module"] if "module" in data_dict else "no_module"
        data_type = data_dict["data_type"] if "data_type" in data_dict else "no_data_type"
        return Data(service, version, module, data_type)


class DataUsage:

    def __init__(self):
        self.data_usage_store = {}

    def set_usage(self, usage: int, service: str, version: str, module: str, data_type: str):
        if service not in self.data_usage_store:
            self.data_usage_store[service] = {}
        if version not in self.data_usage_store[service]:
            self.data_usage_store[service][version] = {}
        if module not in self.data_usage_store[service][version]:
            self.data_usage_store[service][version][module] = {}
        self.data_usage_store[service][version][module][data_type] = usage

    def get_usage(self, service: str, version: str, module: str, data_type: str) -> int:
        result = 0
        if(service in self.data_usage_store
                and version in self.data_usage_store[service]
                and module in self.data_usage_store[service][version]
                and data_type in self.data_usage_store[service][version][module]):
            result = self.data_usage_store[service][version][module][data_type]
        return result

    def set_usage_with_data(self, usage: int, data: Data):
        self.set_usage(usage, data.service, data.version, data.module, data.data_type)

    def get_usage_with_data(self, data: Data) -> int:
        return self.get_usage(data.service, data.version, data.module, data.data_type)

