from typing import List
from .data_util import DataUsage
from .hive_log_parser import QueryHistory
import os

out_dir = "./out"


def make_out_dir(sub_path: str) -> str:
    path = f"{out_dir}/{sub_path}"
    os.makedirs(name=path, exist_ok=True)
    return path


def export_unused_items_to_checklist(data_usage: DataUsage, sub_path: str):
    checklist = []
    for service in data_usage.data_usage_store.keys():
        for version in data_usage.data_usage_store[service].keys():
            for module in data_usage.data_usage_store[service][version].keys():
                if not data_usage.get_usage(service, version, module, "total"):
                    checklist.append(f"{service}.{module}_{version}")
    checklist = sorted(checklist)
    path = make_out_dir(sub_path)
    with open(f"{path}/checklist.md", "w") as fs:
        for item in checklist:
            fs.write("- [x] {0}\n".format(item))


def export_data_usage_to_table(
        data_usage: DataUsage, columns: List[str], sorting_columns: List[str], sub_path: str):
    path = make_out_dir(sub_path)
    with open(f"{path}/table.md", "w") as fs:
        for service in data_usage.data_usage_store.keys():
            fs.write(f"### {service}\n\n")
            for version in sorted(data_usage.data_usage_store[service].keys()):
                fs.write(f"#### {version}\n\n")
                for sorting_column in sorting_columns:
                    sorted_modules = sorted(
                        data_usage.data_usage_store[service][version].keys(),
                        key=lambda x: data_usage.get_usage(service, version, x, sorting_column)
                    )
                    fs.write('<details markdown = "1">\n')
                    fs.write(f"<summary> {sorting_column} 기준으로 정렬(펼치기) </summary>\n\n")
                    legend = f"|module|"
                    border = "|---|"
                    for column in columns:
                        legend += f"{column}|"
                        border += "---|"
                    fs.write(f"{legend}\n")
                    fs.write(f"{border}\n")
                    for module in sorted_modules:
                        row = f"|{module}|"
                        for column in columns:
                            usage = data_usage.get_usage(service, version, module, column)
                            row += f"{usage}|"
                        fs.write(f"{row}\n")
                    fs.write("</details>\n\n")


def export_query_histories_to_table(query_histories: List[QueryHistory], sub_path: str):
    path = make_out_dir(sub_path)
    with open(f"{path}/query_history.md", "w") as fs:
        fs.write("## Query History\n")
        container = {}
        for query_history in query_histories:
            if query_history.principal not in container:
                container[query_history.principal] = {}
            container[query_history.principal][query_history.timestamp] = {"query": query_history.raw_hiveql,
                                                                           "tables": query_history.tables}
        for principal in sorted(container.keys()):
            fs.write(f"### {principal}\n")
            fs.write('<details markdown = "1">\n')
            fs.write(f"<summary>펼치기</summary>\n\n")
            fs.write("|timestamp|query|\n")
            fs.write("|---|---|\n")
            for timestamp in sorted(container[principal].keys()):
                query = container[principal][timestamp]["query"].replace("\n", " ")
                fs.write(f"|{timestamp}|{query}|\n")
            fs.write("</details>\n\n")
