from pyhive import hive
import pandas
from typing import List
import thrift
import base64
import thrift.transport.THttpClient


def thrift_http_transport(host: str, username: str, password: str):
    transport = thrift.transport.THttpClient.THttpClient(
        uri_or_host=host
    )

    auth_credentials = f'{username}:{password}'.encode('UTF-8')
    auth_credentials_base64 = base64.standard_b64encode(auth_credentials).decode('UTF-8')
    transport.setCustomHeaders(
        {
            'Authorization': f'Basic {auth_credentials_base64}'  # HiveServer2 BASIC auth
        }
    )
    return transport


def query_to_hive_with_http_transport(host: str, username: str, password: str, hiveql_list: List[str]):
    conn = hive.connect(thrift_transport=thrift_http_transport(host, username, password))
    cursor = conn.cursor()
    for hiveql in hiveql_list:
        cursor.execute(hiveql)
    query_results = cursor.fetchall()
    cursor.close()
    conn.close()

    return query_results


def query_to_hive(host: str, username: str, password: str, hiveql: str, database="default"):
    conn = hive.Connection(host=host, auth='CUSTOM', port=10000, username=username,
                           password=password, database=database)
    query_result = pandas.read_sql_query(hiveql, conn)
    return query_result
