import pytest
import hive_log_parser as parser


@pytest.fixture
def test_queries():
    return [
        """SELECT
    to_json( named_struct(
    'visits_id', visits.id,
    'idno', visits.`user`.unique_id,
    'gender', users.info.gender,
    'age', (year(current_date) - users.info.birthyear),
    'ages', cast((floor((year(current_date) - users.info.birthyear)/10) * 10) as string),
    'doname', nvl(get_json_object(users.info.payaddress, '$.doName'), '-'),
    'siname', nvl(get_json_object(users.info.payaddress, '$.siName'), '-'),
    'receipt_id', nvl(receipt.id, '-'),
    'receipt_url', nvl(receipt.url, '-'),
    'consume_day', substring_index(`timestamp`, ' ', 1),
    'consume_time', substring_index(`timestamp`, ' ', -1),
    'menu1', nvl(receipt.items[0].name, '-'),
    'unit_count1', nvl(cast(receipt.items[0].count as INT), '-'),
    'unit_price1', nvl(cast(receipt.items[0].price as INT), '-'),
    'menu2', nvl(receipt.items[1].name, '-'),
    'unit_count2', nvl(cast(receipt.items[1].count as INT), '-'),
    'unit_price2', nvl(cast(receipt.items[1].price as INT), '-'),
    'menu3', nvl(receipt.items[2].name, '-'),
    'unit_count3', nvl(cast(receipt.items[2].count as INT), '-'),
    'unit_price3', nvl(cast(receipt.items[2].price as INT), '-'),
    'menu4', nvl(receipt.items[3].name, '-'),
    'unit_count4', nvl(cast(receipt.items[3].count as INT), '-'),
    'unit_price4', nvl(cast(receipt.items[3].price as INT), '-'),
    'menu5', nvl(receipt.items[4].name, '-'),
    'unit_count5', nvl(cast(receipt.items[4].count as INT), '-'),
    'unit_price5', nvl(cast(receipt.items[4].price as INT), '-'),
    'total_price', receipt.total_price,
    'place_id', nvl(places.id, '-'),
    'category1', nvl(places.category.categorycodes[0], '-'),
    'category2', nvl(places.category.categorycodes[1], '-'),
    'category3', nvl(places.category.categorycodes[2], '-'),
    'category4', nvl(places.category.categorycodes[3], '-'),
    'place_address', nvl(places.address, '-'),
    'review_id', nvl(reviews.id, '-'),
    'photo_number', nvl(size(reviews.photo_number), 0),
    'rating', nvl(reviews.rating, '-'),
    'text_review', nvl(reviews.body, '-'),
    'visit_count', nvl(visits.visit_count, '-'),
    'review_theme', review_group_by.themes,
    'review_media', review_group_by.media,
    'votedkeyword', reviews.voted_keywords,
    'analysis', reviews.analysis))
    FROM (
    SELECT  id,
    evidences[0] as receipt_evidence,
    evidences[1] as review_evidence,
    `user`,
    place_id,
    substring_index(cast(from_utc_timestamp(visit_date_time, 'KST') as STRING), '.', 1) as `timestamp`,
    statistic.visit_count as visit_count
    FROM snapshot.pup_visit_v2
    WHERE
    evidences[0].type = 'RECEIPT'
    AND place_id != ''
    AND year = 2022 AND month = 05 AND day = 04
    AND from_utc_timestamp(cast(created_date_time as BIGINT), 'GMT') > '2022-05-04'
    ) AS visits
    LEFT OUTER JOIN (
    SELECT  id,
    image.url as url,
    items,
    total_price
    FROM snapshot.pup_receipt_v2
    WHERE year = 2022 AND month = 05 AND day = 04
    ) AS receipt ON (visits.receipt_evidence.id = receipt.id)
    LEFT OUTER JOIN (
    SELECT  fulldocument.idno as id,
    fulldocument.info as info
    FROM snapshot.myplace_myplaceusers_v1
    WHERE
    length(fulldocument.idno) > 0
    and cast(fulldocument.info.birthyear AS int) != 0
    and year = 2022 and month = 05 and day = 04
    ) AS users ON (visits.`user`.unique_id = users.id)
    LEFT OUTER JOIN (
    SELECT  fulldocument.id as id,
    fulldocument.category as category,
    fulldocument.address as address
    FROM snapshot.myplace_places_v1
    WHERE year = 2022 and month = 05 and day = 04
    ) AS places ON (visits.place_id = places.id)
    LEFT OUTER JOIN (
    SELECT  id,
    review_ids[0] as review_id
    FROM snapshot.pup_review_group_v2
    WHERE year = 2022 AND month = 05 AND day = 04
    ) AS review_group ON (visits.review_evidence.type = 'REVIEW_GROUP' and review_group.id = visits.review_evidence.id)
    LEFT OUTER JOIN (
    SELECT  id,
    collect_list(analysis.themes) as themes,
    collect_list(content.media_items) as media
    FROM snapshot.pup_review_v2
    WHERE year = 2022 AND month = 05 AND day = 04
    GROUP BY id
    ) AS review_group_by ON (review_group.review_id = review_group_by.id)
    LEFT OUTER JOIN (
    SELECT  id,
    content.media_items as photo_number,
    content.rating as rating,
    content.text as body,
    analysis,
    content.tags.voted_keywords as voted_keywords
    FROM snapshot.pup_review_v2
    WHERE year = 2022 AND month = 05 AND day = 04
    ) as reviews ON (review_group.review_id = reviews.id)
""",
        """SELECT 
    to_json(named_struct(
    'idhash', users.fulldocument.idno,
    'gender', users.fulldocument.info.gender,
    'age', cast( cast((year(current_date) - users.fulldocument.info.birthyear) as int) as string),
    'age_group', cast((floor((year(current_date) - users.fulldocument.info.birthyear)/10) * 10) as string),
    'isrealname', lower(cast((users.fulldocument.info.isrealname) as string)),
    'grade', users.fulldocument.currentgrade,
    'review_totalcount', nvl(userstats.fulldocument.review.totalcount, '0'),
    'review_imagecount', nvl(userstats.fulldocument.review.imagecount, '0'),
    'good_imagecount', nvl(userstats.fulldocument.review.goodimagecount, '0'),
    'good_image_reviewcount', nvl(userstats.fulldocument.review.goodimagereviewcount, '0'),
    'image_reviewcount', nvl(userstats.fulldocument.review.imagereviewcount, '0'),
    'following_count', nvl(place_following.following_number, '0'),
    'follower_count', nvl(follower.cnt, '0'),
    'following_userlist', place_following.followings.id,
    'application_id', nvl(userstats.fulldocument.application.id, '-'),
    'documentkey', users.documentkey,
    'updated_time', regexp_replace(substring_index(cast(from_utc_timestamp(from_unixtime(floor(users.fulldocument.updated.`$date` / 1000)), 'yyyyMMdd') as STRING), ' ', 1), '-', '')
    ))
    FROM
    (SELECT fulldocument, documentkey.`_id`.`$oid` as documentkey
    FROM myplace_myplaceusers_v1
    WHERE year = 2022 and month = 04 and day = 22
    ) AS users
    LEFT OUTER JOIN
    (SELECT fulldocument
    FROM myplace_userstats_v1
    WHERE year = 2022 and month = 04 and day = 22
    ) AS userstats
    ON users.fulldocument.idno = userstats.fulldocument.id
    LEFT OUTER JOIN
    (SELECT size(fulldocument.followings) as following_number, fulldocument.userid as userid, fulldocument.followings as followings
    FROM fever_place_following_v1
    WHERE year = 2022 and month = 04 and day = 22
    ) AS place_following
    ON users.fulldocument.idno = place_following.userid
    LEFT OUTER JOIN
    (SELECT fulldocument.followings[0].id as userid, count(*) as cnt
    FROM fever_place_following_v1
    WHERE year = 2022 and month = 04 and day = 22
    GROUP BY fulldocument.followings[0].id
    ) AS follower
    ON users.fulldocument.idno = follower.userid
    WHERE
    users.fulldocument.idno is not null
    and users.fulldocument.currentgrade is not null
""",
    ]


@pytest.fixture
def tables_of_test_queries():
    return [
        ['snapshot.pup_visit_v2', 'snapshot.pup_receipt_v2', 'snapshot.myplace_myplaceusers_v1',
         'snapshot.myplace_places_v1', 'snapshot.pup_review_group_v2', 'snapshot.pup_review_v2'],
        ['myplace_myplaceusers_v1', 'myplace_userstats_v1', 'fever_place_following_v1']
    ]


@pytest.fixture
def setting_test_queries():
    return [
        """set hive.execution.engine=tez
SELECT *
FROM `compact`.`booking_abuse_relation_v1`
WHERE ( 
`compact_booking_abuse_relation_v1`.`day`=11 
AND ( `compact_booking_abuse_relation_v1`.`year`=2022
AND `compact_booking_abuse_relation_v1`.`month`=5 )
)
""",
        """SET hive.execution.engine=tez
SELECT *
FROM `compact`.`booking_abuse_relation_v1`
WHERE ( 
`compact_booking_abuse_relation_v1`.`day`=11 
AND ( `compact_booking_abuse_relation_v1`.`year`=2022
AND `compact_booking_abuse_relation_v1`.`month`=5 )
)
""",
        """  set   hive.execution.engine=tez
SELECT *
FROM `compact`.`booking_abuse_relation_v1`
WHERE ( 
`compact_booking_abuse_relation_v1`.`day`=11 
AND ( `compact_booking_abuse_relation_v1`.`year`=2022
AND `compact_booking_abuse_relation_v1`.`month`=5 )
)
"""
    ]


def test_get_tables_from_query(test_queries, tables_of_test_queries):
    for i, query in enumerate(test_queries):
        result = parser.QueryHistory.from_lines(query.split("\n"), "", "", "")
        assert result.tables == tables_of_test_queries[i]


def test_setting_hive_config(setting_test_queries):
    for query in setting_test_queries:
        result = parser.QueryHistory.from_lines(query.split("\n"), "", "", "")
        assert "hive.execution.engine" in result.settings
        assert result.settings["hive.execution.engine"] == "tez"
