import option_util
import pytest


@pytest.fixture
def default_args():
    return option_util.get_parser_with_default_args()


def test_default_option(default_args):
    args = vars(default_args.parse_args([
        "--hive-username=user",
        "--hive-password=pass"
    ]))
    # default num_threads is 2
    assert args.get("num_threads") == 2


def test_required_option(default_args):
    try:
        default_args.parse_args()
        assert False
    except SystemExit:
        assert True


def test_setting_option(default_args):
    default_args.add_argument("--test-str", type=str)
    default_args.add_argument("--test-int", type=int)
    default_args.add_argument("--test-bool", type=bool)

    args = vars(default_args.parse_args([
        "--hive-username=user",
        "--hive-password=pass",
        "--test-str=test",
        "--test-int=777",
        "--test-bool=True"
    ]))

    assert args.get("test_str") == "test"
    assert args.get("test_int") == 777
    assert args.get("test_bool")
